#ifndef __ASSOCIATION_ANALYSIS_H__
#define __ASSOCIATION_ANALYSIS_H__

#include "VarianceComponents.h"
#include "GenotypeInference.h"
#include "MerlinKinship.h"

class FamilyAnalysis;

class AssociationAnalysis : protected VarianceComponents
   {
   public:
      virtual ~AssociationAnalysis() {}

      void AnalyseAssociation(FamilyAnalysis & engine, bool threeModels = false);
      void AnalyseAssociation2df(FamilyAnalysis & engine);
      void PolygenicModel(Pedigree & ped, int t, NormalSet & mvn, IntArray * pheno, bool meanCenterCovariates = false);

      static void OutputRefinedModels(const String & prefix, Pedigree & ped);

      void OpenPerFamilyFile(int chromosome);
      void ClosePerFamilyFile(int chromosome);

   protected:
      virtual void SetupPDF(MerlinPDF & pdf, String & trait, bool threeModels = false);

      int  RetrievePhenotypes(FamilyAnalysis & engine, int modelIndex, String & traitLabel, IntArray * pheno, int & families);

      int  SuggestDigitsForEffectSize(double effect);
      int  CountVarianceComponents();
      void ReportCovariates();

   private:
      // This variable is static so it can be updated across chromosomes
      static RefinedQtlModel refinedAssociationModels;

   };

#endif


