#ifndef __HAPLOGRAPH_H__
#define __HAPLOGRAPH_H__

#include "HaploFamily.h"
#include "Unknown.h"

#include <stdio.h>

class HaplotypeGraph
   {
   public:
      // Number of haplotypes in this graph
      int count;

      // Allele state for each marker
      IntArray * haplotypes;

      // Index of missing genotypes and other ambiguities
      SetOfUnknowns unknowns;

      // Index of last genotyped marker for each haplotype
      IntArray lastGenotype;

      // Relative weight for this graph
      double weight;

      // Probability of observing this haplotype set
      double likelihood;

      // Scale factor for the likelihood, in 2^-128 units
      int    scale;

      HaplotypeGraph()
         { haplotypes = NULL; count = 0;}

      ~HaplotypeGraph()
         { if (haplotypes != NULL) delete [] haplotypes; }

      void LoadFromFile(FILE * file, int haplos);
      void LoadFromMemory(FounderGraph * founderGraph, int haplos, int markers);

      // Number of alternative solutions for this haplotype graph
      double Complexity(IntArray & alleleCounts);

   private:
      void Dimension(int size, int markers);
   };

#endif


